/**
 * 
 */
package edu.illinois.ncsa.paw.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;

import edu.illinois.ncsa.domain.AbstractBean;
import edu.illinois.ncsa.domain.Person;

/**
 * Definition of an active workflow published to the web. Only specific inputs,
 * outputs and parameters are exposed on the web. Input widgets and output
 * visualization are configured to meaningfully expose inputs and visualize
 * outputs.
 * 
 * @author Luigi Marini
 * 
 */
@Entity(name = "PublishedWorkflow")
@Document(collection = "PublishedWorkflow")
public class PublishedWorkflow extends AbstractBean {

    /** Pretty name **/
    private String                           title;

    /** Long description **/
    private String                           description;

    /** Date when it was created **/
    @Temporal(TemporalType.TIMESTAMP)
    private Date                             dateCreated;

    /** User who created it **/
    @OneToOne(fetch = FetchType.EAGER, cascade = { CascadeType.MERGE })
    private Person                           creator;

    /** Underlying workflow **/
    private String                           workflowId;

    /** List of configured input widgets **/
    @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE })
    @JoinTable(name = "PWInputConfigurations")
    private List<InputConfiguration>         inputs           = new ArrayList<InputConfiguration>();

    /** List of configured visualizations **/
    @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE })
    @JoinTable(name = "PWVisualizations")
    private List<VisualizationConfiguration> visualizations;

    @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE })
    @JoinTable(name = "PWSubmissionSets")
    @DBRef
    private List<PWSubmissionSet>              submissions;

    /** Used for serialization of object */
    private static final long                serialVersionUID = 1L;

    public PublishedWorkflow() {}

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Date getDateCreated() {
        return dateCreated;
    }

    public void setDateCreated(Date dateCreated) {
        this.dateCreated = dateCreated;
    }

    public Person getCreator() {
        return creator;
    }

    public void setCreator(Person creator) {
        this.creator = creator;
    }

    public String getWorkflowId() {
        return workflowId;
    }

    public void setWorkflowId(String workflowId) {
        this.workflowId = workflowId;
    }

    public List<InputConfiguration> getInputs() {
        return inputs;
    }

    public void setInputs(List<InputConfiguration> inputWidgets) {
        this.inputs = inputWidgets;
    }

    public void addInput(InputConfiguration inputConfiguration) {
        this.inputs.add(inputConfiguration);
    }

    public List<VisualizationConfiguration> getVisualizations() {
        return visualizations;
    }

    public void setVisualizations(List<VisualizationConfiguration> visualizations) {
        this.visualizations = visualizations;
    }

    public void addVisualization(VisualizationConfiguration visualizationConfiguration) {
        this.visualizations.add(visualizationConfiguration);
    }

    public List<PWSubmissionSet> getExecutions() {
        return submissions;
    }

    public void setExecutions(List<PWSubmissionSet> executions) {
        this.submissions = executions;
    }

    public void addExecution(PWSubmissionSet executionSet) {
        this.submissions.add(executionSet);
    }
}
